use std::borrow::Cow;

use hyper::StatusCode;
use tracing::warn;
use warp::{
    hyper,
    reject::{Reject, Rejection},
    reply::Response,
};

#[derive(Debug)]
pub struct InternalServerError(pub Cow<'static, str>);
impl Reject for InternalServerError {}

#[derive(Debug)]
pub struct NotFoundError(pub Cow<'static, str>);
impl Reject for NotFoundError {}

pub async fn handle_rejection(err: Rejection) -> Result<Response, std::convert::Infallible> {
    let (status_code, body): (StatusCode, Cow<'static, str>) =
        if err.is_not_found() || err.find::<NotFoundError>().is_some() {
            (
                StatusCode::NOT_FOUND,
                err.find::<NotFoundError>()
                    .map_or("".into(), |reason| reason.0.clone()),
            )
        } else if let Some(InternalServerError(reason)) = err.find::<InternalServerError>() {
            (StatusCode::INTERNAL_SERVER_ERROR, reason.clone())
        } else {
            warn!("unhandled rejection: {:?}", err);
            (StatusCode::INTERNAL_SERVER_ERROR, "".into())
        };

    Ok(hyper::Response::builder()
        .header("Content-Type", "text/plain")
        .status(status_code)
        .body(body.into())
        .expect("building the response should never fail"))
}
